using System;
using JetBrains.Annotations;
using UniRx;

namespace Magify
{
    public class BasePurchaserService : IDisposable
    {
        [CanBeNull]
        public IMinimalInAppStore InAppStore { get; private set; }
        [NotNull]
        public IReactiveProperty<IPurchaseVerificationHandler> ExternalPurchaseVerificationHandler => MagifyManager.ExternalPurchaseVerificationHandler;
        [NotNull]
        public FloatReactiveProperty VerificationRetryInterval => MagifyManager.VerificationRetryInterval;

        public virtual void SetPurchasingProvider([CanBeNull] IMinimalInAppStore inAppStore)
        {
            DisposeCurrentInAppStore();
            InAppStore = inAppStore;
            if (InAppStore != null)
            {
                InAppStore.OnPurchaseFinished += ExternalPurchaseFinishedHandler;
            }
        }

        private void DisposeCurrentInAppStore()
        {
            if (InAppStore != null)
            {
                InAppStore.OnPurchaseFinished -= ExternalPurchaseFinishedHandler;
            }
        }

        void IDisposable.Dispose()
        {
            DisposeCurrentInAppStore();
        }

        private void ExternalPurchaseFinishedHandler(string productId, PurchaseInfo purchaseInfo)
        {
            if (purchaseInfo.SubscriptionInfo == null)
            {
                TrackInApp(productId, purchaseInfo);
            }
        }

        private void TrackInApp(string productId, PurchaseInfo purchaseInfo)
        {
            MagifyManager.TrackExternalInAppFor(
                productId,
                purchaseInfo.Price,
                purchaseInfo.Currency,
                purchaseInfo.TransactionId,
                purchaseInfo.PurchaseToken,
                purchaseInfo.OriginalTransactionId,
                purchaseInfo.Receipt.Payload);
        }
    }
}