﻿using System.Threading;
using System.Threading.Tasks;
using JetBrains.Annotations;

namespace Magify
{
    internal static class ThreadingUtils
    {
        public static void CancelAndReplace(ref CancellationTokenSource cts)
        {
            var old = cts;
            cts = new CancellationTokenSource();
            old?.Cancel();
            old?.Dispose();
        }

        public static void CancelAndReplace<T>(ref CancellationTokenSource cts, T newValue)
            where T: CancellationTokenSource
        {
            var old = cts;
            cts = newValue;
            old?.Cancel();
            old?.Dispose();
        }

        public static void CancelAndRemove(ref CancellationTokenSource cts)
        {
            var old = cts;
            cts = null;
            old?.Cancel();
            old?.Dispose();
        }

        public static void TrySetResultAndRemove<T>(ref TaskCompletionSource<T> completionSource, [CanBeNull] T result)
        {
            var old = completionSource;
            completionSource = null;
            old?.TrySetResult(result);
        }

        public static void TryCancelAndRemove<T>(ref TaskCompletionSource<T> completionSource)
        {
            var old = completionSource;
            completionSource = null;
            old?.TrySetCanceled();
        }

        public static void TryCancelAndRemove<T>(ref TaskCompletionSource<T> completionSource, CancellationToken cancellationToken)
        {
            var old = completionSource;
            completionSource = null;
            old?.TrySetCanceled(cancellationToken);
        }
    }
}