﻿using System.Threading.Tasks;
using Cysharp.Threading.Tasks;
using NUnit.Framework;

namespace Magify.Tests
{
    internal partial class MagifyClientTests
    {
        public class InfinitiveLoops : MagifyClientTestsPart
        {
            [Test]
            [TestCase(true, 1)]
            [TestCase(false, 1)]
            [TestCase(true, 30)]
            [TestCase(false, 30)]
            [TestCase(true, 100)]
            [TestCase(false, 100)]
            public async Task WhenMagifyClientHotStarted_ThenSetup_ThenNoInfinitiveLoop(bool status, int delayMs)
            {
                // Arrange offline
                using (CreateDefaultAndInit())
                {
                }

                // Arrange online
                var network = new NetworkMoq(async (_, m, ct) =>
                {
                    await UniTask.Delay(delayMs, cancellationToken: ct);
                    return NetworkMoq.OkResult(m);
                });
                using var magifyClient = CreateDefault(networkClient: network, environment: Environment.Staging);
                // Act
                magifyClient.InitializeSdk();
                magifyClient.Setup();

                await UniTask.Delay(delayMs + delayMs / 10, cancellationToken: GetCancellationToken());

                // Assert
                Assert.Pass();
            }

            [Test]
            [TestCase(true, 1)]
            [TestCase(false, 1)]
            [TestCase(true, 30)]
            [TestCase(false, 30)]
            [TestCase(true, 100)]
            [TestCase(false, 100)]
            public async Task WhenMagifyClientHotStarted_ThenSetStatusesAndSetup_ThenNoInfinitiveLoop(bool status, int delayMs)
            {
                // Arrange offline
                using (CreateDefaultAndInit())
                {
                }

                // Arrange online
                var network = new NetworkMoq(async (_, m, ct) =>
                {
                    await UniTask.Delay(delayMs, cancellationToken: ct);
                    return NetworkMoq.OkResult(m);
                });
                using var magifyClient = CreateDefault(networkClient: network, environment: Environment.Staging);
                // Act
                magifyClient.InitializeSdk();
                magifyClient.SubscriptionStatus.Value = SubscriptionStatus.Paid;
                magifyClient.AuthorizationStatus.Value = AuthorizationStatus.Authorized;
                magifyClient.IsGdprGranted.Value = status;
                magifyClient.SetAttStatus(status);

                magifyClient.Setup();

                await UniTask.Delay(delayMs + delayMs / 10, cancellationToken: GetCancellationToken());

                // Assert
                Assert.Pass();
            }
        }
    }
}